/**
 * Title: Introspection
 * Description:
 * Copyright:    Copyright (c) 2009
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.webapp.services ;

import java.util.* ;

import javax.xml.namespace.QName ;

import org.apache.axiom.om.OMElement ;
import org.apache.axiom.soap.SOAPBody ;
import org.apache.axiom.soap.SOAPEnvelope ;

import org.apache.axis2.context.* ;
import org.apache.axis2.description.AxisMessage ;
import org.apache.axis2.description.AxisService ;
import org.apache.axis2.description.AxisOperation ;
import org.apache.axis2.description.AxisDescription ;
import org.apache.axis2.addressing.EndpointReference ;

/**
* This web service return some of the configuration parameters offered by the different <code>context</code>
* objects handled by the Axis2 framework. The web service is invoked by the <code>test_axis2.jsp</code> page
* using the <code>ClientThread</code> class, in order to call the web service using different threads
* (to make sure that Axis2 must handle the service calls simultaneously).
*/
public class Introspection {

	/** Specify if this web service must log detailed info on the STDOUT.								*/
	protected boolean verbose = true ;

	/** The value of the input string used to force a delay when producing the response.				*/
	protected static final String WAIT = "wait" ;

	/** The leading name of each message logged on the STDOUT.											*/
	protected String leadingName = "Introspection - " ;

	/**
	* Simple version of the main operation, returning all the introspection parameters within an
	* unique string element.<br/>
	* <strong>Remark:</strong> if the input parameter starts with "wait" this method waits about 1 sec
	* before returning the SOAP response.
	*
	* @param msg a simple input message (used just to echo the input text).
	* @return a text message containing the result of the introspection calls.
	*/
    public String getContextInfo(String input) {

		StringBuffer result = getBuffer(input) ;
		if (verbose) System.out.println(leadingName + "Input (info): " + input) ;
		if ( input.startsWith(WAIT) ) this.delay() ;

		if (result != null) return result.toString() ; else return "no data" ;
    }

	/**
	* The main operation offered by this service, returning the introspection parameters as a
	* <code>String[]</code> array, hence the SOAP response will be an array of "ns:return" XML elements.<br/>
	* <strong>Remark:</strong> if the input parameter starts with "wait" this method waits about 1 sec
	* before returning the SOAP response.
	*
	* @param msg a simple input message (used just to echo the input text).
	* @return a text message containing the result of the introspection calls.
	*/
    public String[] getContextList(String input) {

		String[] result ;
		StringBuffer[] array = getArray(input) ;
		if (verbose) System.out.println(leadingName + "Input (list): " + input) ;
		if ( input.startsWith(WAIT) ) this.delay() ;

		if (array != null) {
			int max = array.length ;
			result = new String[max] ;
			for (int i = 0 ; i < max ; i++) result[i] = array[i].toString() ;
		} else {
			result = new String[] { "no data" } ;
		}

		return result ;
    }

// **************************************************************************************************************
// **********************************			PRIVATE METHODS			 ****************************************
// **************************************************************************************************************

	/**
	* Methos used by <code>getContextInfo()</code> to produce the introspection single string response.
	*
	* @param msg a simple input message (used just to echo the input text).
	* @return a text message containing the result of the introspection calls.
	*/
    private StringBuffer getBuffer(String input) {

		String eol = "   " ;
		StringBuffer[]	array	= getArray(input) ;
		StringBuffer	result	= new StringBuffer("") ;
		
		if (array != null) for (int i = 0 ; i < array.length ; i++) {
			result.append(array[i] + eol) ;
		}
		return result ;
	}

	/**
	* Methos used by <code>getContextList()</code> to produce the introspection array response.
	*
	* @param msg a simple input message (used just to echo the input text).
	* @return a text message containing the result of the introspection calls.
	*/
    private StringBuffer[] getArray(String input) {

		// Creating result array
		int max = 42 ;
		StringBuffer[] result = new StringBuffer[max] ;
		for (int i = 0 ; i < max ; i++) result[i] = new StringBuffer("none") ;

		// Setting general service data
		int lastIndex = 0 ;		
		result[0] = new StringBuffer("Echo " + input) ;
		result[1] = new StringBuffer("Service object: " + this + " [" + getThreadID() + ']') ;

		// Reading Axis2 objects
		MessageContext 			messageContext 			= MessageContext.getCurrentMessageContext() ;
		SOAPEnvelope			soapEnvelope 			= messageContext.getEnvelope() ;
		AxisMessage 			axisMessage				= messageContext.getAxisMessage() ;
		AxisOperation 			axisOperation 			= messageContext.getAxisOperation() ;		
		AxisService 			axisService 			= messageContext.getAxisService() ;		
		OperationContext 		operationContext		= messageContext.getOperationContext() ;
		ConfigurationContext	configurationContext	= messageContext.getRootContext() ;
		
		// Retrieving information from the SOAP ENVELOPE
		if (soapEnvelope != null) {
			OMElement element = null ;
			SOAPBody body = soapEnvelope.getBody() ;
			if (body != null) element = body.getFirstElement() ;
			result[2] 	= new StringBuffer("FirstElement: " + element) ;
		}
		
		// Retrieving information from the MESSAGE CONTEXT
		if (messageContext != null) {
			int i = 3 ;		
			EndpointReference toEPR = messageContext.getTo() ;
			EndpointReference fromEPR = messageContext.getFrom() ;
			result[i]    = new StringBuffer("********* MessageContext *********") ;			
			result[i+1]  = new StringBuffer("MessageContextID: "	+ messageContext.getLogCorrelationID() ) ;
			result[i+2]  = new StringBuffer("MessageID: "			+ messageContext.getMessageID() ) ;
			result[i+3]  = new StringBuffer("PhaseIndex: "			+ messageContext.getCurrentPhaseIndex() ) ;
			result[i+4]  = new StringBuffer("HandlerIndex: "		+ messageContext.getCurrentHandlerIndex() ) ;
			result[i+5]  = new StringBuffer("SoapAction: "			+ messageContext.getSoapAction() ) ;
			result[i+6]  = new StringBuffer("ToEPR: "				+ toEPR.getAddress() ) ;
			result[i+7]  = new StringBuffer("FromEPR: "				+ fromEPR.getAddress() ) ;
			result[i+8]  = new StringBuffer("MessageFlow: "			+ messageContext.getFLOW() ) ;
			result[i+9]  = new StringBuffer("OutputWritten: "		+ messageContext.isOutputWritten() ) ;
			result[i+10] = new StringBuffer("ResponseWritten: "		+ messageContext.isResponseWritten()  ) ;
			result[i+11] = new StringBuffer("ServerSide: "			+ messageContext.isServerSide() ) ;
		}

		// Retrieving information from the AXIS MESSAGE
		if (axisMessage != null) {
			int i = 15 ;		
			Object 				key 				= axisMessage.getKey() ;
			QName 				qName 				= axisMessage.getElementQName() ;
			AxisDescription 	axisDescription 	= axisMessage.getChild(key) ;
			result[i]	= new StringBuffer("********* AxisMessage *********") ;
			result[i+1] = new StringBuffer("AxisMessage: " 			+ axisMessage) ;
			result[i+2] = new StringBuffer("MessageDirection: " 	+ axisMessage.getDirection() ) ;
			result[i+3] = new StringBuffer("MessageName: " 			+ axisMessage.getName() ) ;
			result[i+4] = new StringBuffer("QName: " 				+ qName.getLocalPart() ) ;
			result[i+5] = new StringBuffer("ObjectKey: " 			+ key ) ;
			result[i+6] = new StringBuffer("AxisDescription: " 		+ axisDescription ) ;
		}
		
		// Retrieving information from the AXIS OPERATION
		if (axisOperation != null) {
			int i = 22 ;
			result[i] 	= new StringBuffer("********* AxisOperation *********") ;			
			result[i+1] = new StringBuffer("AxisOperation: " 		+ axisOperation) ;
			result[i+2] = new StringBuffer("OperationStyle: " 		+ axisOperation.getStyle() ) ;
			result[i+3] = new StringBuffer("InputAction: " 			+ axisOperation.getInputAction() ) ;
			result[i+4] = new StringBuffer("OutputAction: " 		+ axisOperation.getOutputAction() ) ;
		}

		// Retrieving information from the AXIS SERVICE
		if (axisService != null) {
			int i = 27 ;
			result[i] 	= new StringBuffer("********* AxisService *********") ;			
			result[i+1] = new StringBuffer("AxisService: " 			+ axisService) ;
			result[i+2] = new StringBuffer("ServiceScope: " 		+ axisService.getScope() ) ;
			result[i+3] = new StringBuffer("BindingName: "	 		+ axisService.getBindingName() ) ;
			result[i+4] = new StringBuffer("Description: " 			+ axisService.getServiceDescription() ) ;
		}
		
		// Retrieving information from the OPERATION CONTEXT
		if (operationContext != null) {
			int i = 32 ;
			Iterator iterator = null ;
			HashMap hashMap = operationContext.getMessageContexts() ;
			if (hashMap != null) { 	
				Collection collection = hashMap.values() ;		
				iterator = collection.iterator() ;
			}
			result[i] 	= new StringBuffer("********* OperationContext *********") ;			
			result[i+1] = new StringBuffer("OperationContext: " 	+ operationContext) ;
			result[i+2] = new StringBuffer("IsComplete: "	 		+ operationContext.isComplete() ) ;
			result[i+3] = new StringBuffer("OperationName: " 		+ operationContext.getOperationName() ) ;
			lastIndex = i + 4 ;
			// Retrieving all message context bound to this operation
			if (iterator != null) while ( iterator.hasNext() ) {
				MessageContext context = (MessageContext) iterator.next() ;
				result[lastIndex++] = new StringBuffer("MessageContext: " + context.getLogCorrelationID() ) ;
			}
		}
		
		// Retrieving information from the CONFIGURATION CONTEXT
		if (configurationContext != null) {
			int i = lastIndex ;
			result[i] 	= new StringBuffer("********* ConfigurationContext *********") ;						
			result[i+1]	= new StringBuffer("ConfigurationContext: "	+ configurationContext) ;
			result[i+2] = new StringBuffer("ContextRoot: "			+ configurationContext.getContextRoot() ) ;
			result[i+3] = new StringBuffer("ServiceContextPath: "	+ configurationContext.getServiceContextPath() ) ;
		}

		return result ;
	}

// **************************************************************************************************************
// **********************************			 UTILITIES			 ********************************************
// **************************************************************************************************************

	/**
	* Used to delay the service response for about 1 second.
	* See the <code>test_axis2.jsp</code> for details.
	*/
	public void delay() {

		try {
			Thread.currentThread().sleep(1000) ;
			if (verbose) System.out.println(leadingName + "Sleeped thread ID: " + getThreadID()) ;
		} catch (InterruptedException e) {
			System.err.println(leadingName + "Interrupted while sleeping") ;
		}
	}

	/**
	* Return the reference ID of the thread running this service.
	*
	* @return the ID (as a long integer) of the current thread.
	*/
    public long getThreadID() {

		return Thread.currentThread().getId() ;
	}

	/**
	* Called by the Virtual Machine when this object is destroyed.
	*/
	public void finalize() {

		System.out.println(leadingName + "Finalized: " + this) ;
	}

} // END OF CLASS